; Horton Conway's Game of life publish in OSI Small System Journal
; originally for OSI440 video systems
; Based on Horton Conway's game of life described in the 
; "Mathematical Games" department of Scientific American during
; the winter of 1970-1971

; This program has been modified to run on the OSI C2/C4/C8 video systems
; with 4K RAM (8K for patterns) by Mark Spankus
; Thanks to Bill Dromgoole for providing the original OSI440 Sample Audio Cassette

; Enter program at START ($0230)
; Optional Save patterns by entering program at SETPATTERN ($0A35)

;constants
SSIZE = $D8   ; $D4 on C1
ROWLEN = $40  ; $20 on C1
ROWMASK= $C0	;$E0 on C1
SCRLVAL= $C4    ;$A4 on C1
SCREENLO = $D4  ;$D2 on C1

REDIRECT= $20   ;memory location for input flag

; Horton Conway's Game of life OSI540
*=$0230
	.EXE *        ;A65 emit OSI .lod start address operation
START               jsr INITALL
                    lda #<MENU1
                    jsr S02ef
                    lda #$00
					sta REDIRECT
                    sta $0a
L020c               jsr INECHO
                    cmp #$0d
                    beq L0222
                    clc
                    rol a
                    rol a
                    rol a
                    rol a
L0219 = * + 1       
                    ldy #$04
L021a               rol a
                    rol $0a
                    dey
                    bne L021a
                    beq L020c
L0222               lda #$0a
                    jsr OUTSCRN
                    jmp L0300
                    
L022a               lda #<MENU2
                    JSR S03d0
                    jmp L0238
                    
L0232               JSR POLLKB	;getkey from KB
                    nop
                    bmi L025b
L0238               JSR S03d0
					lda #<MENU2
                    jsr S02ef
                    jsr INECHO
                    cmp #$52
                    beq START
                    cmp #$43
                    beq L025b
                    cmp #$50
                    beq L0238
                    cmp #$45
                    bne L0253
                    jmp $fe00
                    
L0253               lda #$3f
                    nop
                    nop
                    nop
                    jmp L0238
                    
L025b               jsr S040a
                    jsr S0500
                    jsr S03eb
                    jmp L0232
                    
L0267               lda #$06
                    sta $05
                    sta $07
                    lda #$1f
                    sta $06
                    lda #$20
                    sta $04
L0275               lda #$00
                    tay
                    tax
L0279               sta ($00),y
                    sta ($02),y
                    sta ($04,x)
                    sta ($06,x)
                    clc
                    lda $04
                    adc #$20 ;matrix len?
                    sta $04
                    bcc L028d
                    inc $05
                    clc

L028d               lda $06
                    adc #$20 ;matrix len
                    sta $06
                    bcc L0297					
                    inc $07
L0297               iny
                    txa
                    cpy #$1f
                    bne L0279
                    rts
                    
L029e               jsr S041d
                    nop
L02a2               lda ($08),y
                    cmp #$5f
                    beq L02b0
           
                    jsr OUTSCRN
                    inc $08
					BNE *+4
					INC $09
                    jmp L02a2
                    
L02b0               rts
                    
INHEX               jsr S02c3 ;input from keyboad or mem
                    asl a
                    asl a
                    asl a
                    asl a
                    sta $13
                    jsr S02c3
                    and #$0f
                    clc
                    adc $13
                    rts
                    

;either input from keyboard with echo, or stored pattern memory
S02c3               BIT REDIRECT
                    BPL KEYIN
					jsr LOADPAT
					jmp L0a20
KEYIN					
					jsr INECHO
                    jmp L0a20
                    
                    brk 
L02ca               cmp #$30
                    bmi S02c3
                    cmp #$47
                    bpl S02c3
                    cmp #$40
                    bmi L02d9
                    clc
S02d7               adc #$09
L02d9               rts
                    
L02da               tsx
                    inx
                    inx
                    txs
                    jmp L022a
                    
S02e1               clc
                    adc #$30
                    sta ($0d),y
                    iny
                    rts
                    
S02e8               sta ($0d),y
                    inc $0d
                    inc $08
                    rts
                    
S02ef               pha
                    jsr S03d0
                    pla
                    jmp L029e
                    
                    ;.BYTE $00, $00, $00, $00, $00, $00, $00, $00, $00 
L0300               jsr S033c
                    jsr S03ab
					;entry point for interactive pattern entry
L0306               lda #<MENU4
                    jsr S02ef ;output menu4

L030B               ;entry point for non-interactive pattern entry
                    jsr INHEX
                    sta $11
                    jsr INHEX
                    sta $12
                    lda #$06
                    sta $0e
                    lda #$00
                    sta $0d
                    lda $11
L031f               jsr S04ee
                    lda #$1f
                    sec
                    sbc $12
                    tay
                    beq L0332
L032a               lda #$20
                    jsr S0400
                    dey
                    bne L032a 

L0332               lda #$80              
                    sta ($0d),y
                    jsr S0354

L0339               bit REDIRECT
					BMI *+5
					jmp L0306
					jmp L030B
                    
S033c               lda #$05
                    sta $0e
                    lda #$d0
                    jsr S03f1
L0345               tya
                    sta ($0d),y
                    lda #$01
                    jsr S0400
                    lda $0e
                    cmp #$0a
                    bne L0345
                    rts
                    ;copy matrix to screen
S0354               lda #$06               
					sta $0E     ;($0d) is evolution matrix
                    lda #$d0
                    sta $10     ;($0f) is screen coords
                    ldy #$00
                    sty $0f
                    sty $0d
                    ldx #$20
                    jsr S038a
L0367               txa
                    sta ($0f),y
                    lda ($0d),y
                    bpl ROWWRAP
                    lda #$2a      ;cell character
                    sta ($0f),y
					
ROWWRAP             ;this section added to convert C1 screen coords to C2
					lda $0f
					and #$1F
					cmp #$1F
					bne L0372
					lda $0f
					clc
					ADC #ROWLEN/2
					sta $0f
					bcc L0372
					inc $10
					;end of C2 mods				
					
L0372               
					inc $0f
                    bne L0378
                    inc $10
					
L0378               inc $0d
                    bne L0367
                    inc $0e			
				
                    lda #SSIZE ;was #$d4 on C1
                    cmp $10
                    bne L0367
                    jsr S038b	;draw title, evolution counter
                    nop
                    nop
                    nop
S038a               rts
                    
S038b               ldy #$00
                    jsr S03b4
                    nop
                    lda #$d0
                    sta $0e
                    ldx #$01
L0397               lda $0b,x
                    pha
                    lsr a
                    lsr a
                    lsr a
                    lsr a
                    jsr S02e1
                    pla
                    and #$0f
                    jsr S02e1
                    dex
                    bpl L0397
                    rts
                    
S03ab               lda #$00
                    sta $0b
                    sta $0c
                    jmp S0354
                    
S03b4               lda #$d0 ;set screen address of msg
                    sta $0e
                    lda #$84
                    sta $0d
                    lda #$4c ;'L
                    jsr S02e8
                    lda #$49 ;'I
                    jsr S02e8
                    lda #$46 ;'F
                    jsr S02e8
                    lda #$45 ;'E
                    jmp L03f6
                    
S03d0               lda #>MENU2
                    sta $09
                    rts
                    
L03d5               brk
                    
                    brk 
L03d7               lda $0a
L03d9               ldx #$08
L03db               ldy #$ff
L03dd               dey
                    bne L03dd
                    dex
                    bne L03db
                    sed
                    sec
                    sbc #$01
                    cld
                    bne L03d9
                    rts
                    
S03eb               jsr S0354
                    jmp L03d7
                    
S03f1               sta $0d
                    ldy #$00
                    rts
                    
L03f6               jsr S02e8
                    lda #$98
                    sta $0d
                    rts
                    
S0400               clc
                    adc $0d
                    sta $0d
                    bcc L0409
L0407               inc $0e
L0409               rts
                    
S040a               lda #$00
                    sta $00
                    lda #$06
                    sta $01
                    lda #$e1
                    sta $02
                    lda #$09
                    sta $03
                    jmp L0267
                    
S041d               ldy #$00
                    sta $08
                    lda #$80    
                    jsr OUTSCRN
                    lda #$0a
                    jmp OUTSCRN
                    
S042b               lda #$81
                    jmp OUTSCRN
					
                    ;menus must all start on same page!                    
MENU1=*
                    .byte $d, $a, '  -LIFE-', $d, $a
					.byte 'ENTER TIMING CONSTANT',$d,$a
					.byte '(1 - 99): _'
MENU2=*
					.byte $d,$a,$a
					.byte 'AVAILABLE COMMANDS:',$d,$a
					.byte 'C - CONTINUE',$d,$a
					.byte 'E - EXIT',$d,$a
					.byte 'P - PAUSE',$d,$a
					.byte 'R - RESTART',0

MENU4=*
					.byte $80,'INPUT POINTS AS',$d,$a
					.byte 'X,Y ... $ ENDS', $d, $a, '_'
S04ee               pha
                    lda #$0d
                    jsr OUTSCRN
                    pla
                    jmp S0400
                    
S0500               lda #$06
                    sta $0e
                    lda #$00
                    sta $0d
L0508               sec
                    lda $0d
                    sbc #$21
                    sta $0f
                    lda $0e
                    sbc #$00
                    sta $10
                    ldy #$00
                    ldx #$00
                    lda ($0f),y
                    bpl L051e
                    inx
L051e               iny
                    lda ($0f),y
                    bpl L0524
                    inx
L0524               iny
                    lda ($0f),y
                    bpl L052a
                    inx
L052a               ldy #$20
                    lda ($0f),y
                    bpl L0531
                    inx
L0531               iny
                    iny
                    lda ($0f),y
                    bpl L0538
                    inx
L0538               ldy #$40
                    lda ($0f),y
                    bpl L053f
                    inx
L053f               iny
                    lda ($0f),y
                    bpl L0545
                    inx
L0545               iny
                    lda ($0f),y
                    bpl L054b
                    inx
L054b               ldy #$21
                    txa
                    clc
                    adc ($0f),y
                    sta ($0f),y
                    inc $0d
                    bne L0508
                    inc $0e
                    lda $0e
                    cmp #$0a
                    bne L0508
                    lda #$06
                    sta $0e
                    lda #$00
                    sta $0d
                    tay
L0568               lda ($0d),y
                    bpl L057b
                    and #$0f
                    cmp #$04
                    bpl L0576
                    cmp #$02
                    bpl L0581
L0576               tya
                    sta ($0d),y
                    beq L0585
L057b               and #$0f
                    cmp #$03
                    bne L0576
L0581               lda #$80
                    sta ($0d),y
L0585               inc $0d
                    bne L0568
                    inc $0e
                    lda $0e
                    cmp #$0a
                    bne L0568
                    sed
                    clc
                    lda $0b
                    adc #$01
                    sta $0b
                    lda $0c
                    adc #$00
                    sta $0c
                    cld
                    rts
                    

                    

*=$0A00
	.byte $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00

L0a20               cmp #$24  ;check input for end of data
                    bne L0a27
                    jmp L02da
                    
L0a27               cmp #$28  ; if '(' character, redirect input from pattern mem
                    beq L0a2e ; expect 4 hex digits as address
                    jmp L02ca
                    
L0a2e               jsr INHEX ;input with echo 4 character address
                    sta $16
                    jsr INHEX
                    sta $15
					lda #$80
					sta REDIRECT
                    tsx
L0a48               inx
                    inx
                    inx
                    inx
                    txs
                    jmp L0339
SETPATTERN=*                    
L0a50               lda #$28
                    jsr OUTSCRN
                    jsr INHEX
                    sta $16
                    jsr INHEX
                    sta $15
                    ldy #$00
L0a61               jsr INECHO
                    sta ($15),y
                    inc $15
                    bne L0a6c
                    inc $16
L0a6c               cmp #$21
                    bne L0a61
                    jmp START
                    
OUTSCRN             pha          ;screen output
                    sta $f5
                    txa
                    pha
                    tya
                    pha
                    lda $f5
                    beq L0a88
                    bpl L0a8e
                    cmp #$80
                    beq L0aac
                    cmp #$81
                    beq L0ac6
L0a88               pla
                    tay
                    pla
                    tax
                    pla
                    rts
                    
L0a8e               cmp #$0d
                    beq L0b07
                    cmp #$0a
                    beq L0b0c
                    ldy #$00
                    and #$7f
                    sta ($f8),y
                    iny
                    lda ($f8),y
                    sta $fa
                    jsr S0b12
                    lda #$5f
                    ldy #$00
                    sta ($f8),y
                    beq L0a88
L0aac               lda $fa
                    ldy #$00
                    sta ($f8),y
L0ab2               lda #$84
                    sta $f8
                    lda #$d0
                    sta $f9
                    lda ($f8),y
                    sta $fa
                    lda #$5f
                    sta ($f8),y
                    bne L0a88
L0ac4               bne L0a88
L0ac6               ldx #SSIZE
                    lda #$d0
                    sta $f9
                    ldy #$00
                    sty $f8
                    lda #$20
                    sta $fa
L0ad4               sta ($f8),y
                    iny
                    bne L0ad4
                    inc $f9
                    cpx $f9
                    bne L0ad4
                    beq L0ab2
S0ae1               lda $fa
                    ldy #$00
                    sta ($f8),y
                    lda $f8
                    and #ROWMASK ; (#$C0) on C1 is $e0
                    ora #$04
                    sta $f8
                    bne L0afe
L0af1               lda $fa
                    ldy #$00
                    sta ($f8),y
                    lda #ROWLEN
                    jsr S0b14
                    ldy #$00
L0afe               lda ($f8),y
                    sta $fa
                    lda #$5f
                    sta ($f8),y
                    rts
                    
L0b07               jsr S0ae1
                    bne L0ac4
L0b0c               jsr L0af1
                    bne L0ac4
L0b11               rts
                    
S0b12               lda #$01
S0b14               sta $f5
                    clc
                    lda $f8
                    adc $f5
                    sta $f8
                    bcc L0b21
                    inc $f9
L0b21               pha
                    and #$0f
                    cmp #$0c
                    bne L0b33
                    pla
                    and #$10
                    beq L0b34
                    jsr S0ae1
                    jmp L0af1
                    
L0b33               pla
L0b34               lda #$d2  ;cursor @ bottom addr?
                    cmp $f9
                    bpl L0b11
                    lda #$7b
                    cmp $f8
                    bpl L0b11
                    lda $f8    ;yes
                    sec
                    sbc #ROWLEN
                    pha
                    bcs L0b4a
                    dec $f9
L0b4a               lda $f9
                    pha
                    lda #$84 
                    sta $f8
                    lda #SCRLVAL  ;was #$a4 C1
                    sta $f6
                    lda #$d0
                    sta $f7
                    sta $f9
                    ldy #$00
L0b5d               lda ($f6),y
                    sta ($f8),y
                    inc $f6
                    bne L0b67
                    inc $f7
L0b67               inc $f8
                    bne L0b6d
                    inc $f9
L0b6d               lda #$9c
                    cmp $f6
                    bne L0b5d
                    lda #$d3
                    cmp $f7
                    bne L0b5d
                    lda #$80
                    sta $f6
                    lda #$d3
                    sta $f7
                    lda #$20
L0b83               sta ($f6),y
                    iny
                    cpy #$41
                    bne L0b83
                    pla
                    sta $f9
                    pla
                    sta $f8
                    rts
                    
INECHO              jsr $feed
                    jmp OUTSCRN
					
LOADPAT
                    sty $17
                    ldy #$00
                    lda ($15),y
                    inc $15
                    bne L05ad
                    inc $16
L05ad               cmp #$21  ;'!
                    bne L05c6
					lda #$00
					sta REDIRECT
                    tsx
                    inx
                    inx
                    jmp L0a48
                    
L05c6               ldy $17
                    rts					
                    
MACHINE             .BYTE $00
INVKEYB             .BYTE $FF

INITALL             
	SEI                                                 
	LDA	#$00
	STA $DE00     ;set 32x32 mode C4 vid
	BIT	$DF00	  ;check C1/C2-C4
	BMI	C1TYPE
	STA INVKEYB
C1TYPE=*+1
	BIT	$80A9	  ;
	STA	MACHINE	  ;bit 7 = C1, bit 6 = C3, none =C2/C4
    JMP S042b     ;continue with program initialization

POLLKB            ;replacement for ASCII KB test if char ready
	LDA #$FE
	EOR INVKEYB
	STA $DF00
	LDA $DF00
	EOR INVKEYB
	BEQ POLL2
	JSR $FEED	
	AND #$7F
	RTS
POLL2
	LDA #$80
	RTS

;predefined patterns 
* = $1000
Pulsar
	;pulsar cp 45-56-72 $1000
	.byte $30, $45, $31, $30, $30, $46, $31, $30, $30, $43, $31, $30, $30, $44, $31, $30
	.byte $31, $30, $31, $30, $31, $30, $31, $31, $30, $43, $31, $31, $21
Pentomino	
	;R-Pentomino $101D
	.byte $31, $30, $31, $30, $31, $30, $30, $46, $30, $46, $30, $46, $31, $30, $30, $45
	.byte $31, $31, $31, $30, $21
Barber	
	;Barber Pole $1032
	.byte $31, $30, $31, $30, $31, $30, $31, $32, $30, $45, $31, $32, $30, $45, $31, $34
	.byte $30, $43, $31, $34, $30, $43, $31, $36, $30, $42, $31, $37, $30, $41, $31, $37
	.byte $30, $41, $31, $36, $31, $32, $31, $30, $31, $32, $30, $45, $31, $34
	.byte $30, $45, $31, $34, $30, $43, $31, $35, $30, $42, $31, $36, $30, $42, $31, $36
	.byte $30, $43, $21
Figure8	
	;Figure 8 $1073
	.byte $30, $46, $31, $30, $30, $45, $31, $30, $30, $44, $31, $30, $30
	.byte $44, $31, $31, $30, $45, $31, $31, $30, $46, $31, $31, $30, $46, $31, $32, $30
	.byte $45, $31, $32, $30, $44, $31, $32, $31, $30, $30, $46, $31, $31, $30, $46, $31
	.byte $32, $30, $46, $31, $32, $30, $45, $31, $31, $30, $45, $31, $30, $30, $45, $31
	.byte $30, $30, $44, $31, $31, $30, $44, $31, $32, $30, $44, $21 
PI	
	;PI $10BC
	.byte $31, $30, $31, $30, $31, $31, $31, $30, $30, $46, $31, $30, $30, $46, $30, $46
	.byte $30, $46, $30, $45, $31, $31, $30, $46, $31, $31, $30, $45, $21
Glider	
	;Glider $10D9
 	.byte $30, $35, $31, $38, $30, $34, $31, $38, $30, $33, $31, $38, $30, $35, $31, $39 
	.byte $30, $34, $31, $41, $21, $00, $00 
	